﻿var timer = null;
var cometmarks_connect_failed = false;
var cometmarks_local_revision = 0;
var cometmarks = Components.classes["@cometmarks.com/CometMarksExtension;1"].createInstance(Components.interfaces.ICometMarksExtension);

var setting_stateObserver =
{
	observe: function(subject, topic, data)
  {
  	if (topic == "cometmarks-statechange")
		{
			sync_setting_status(data);
			return;
		}
		
		if (topic == "observer_changed")
		{
			setting_register_observer.init();
		}
  }
};

var setting_register_observer =new function()
{
	var cometmarks_gdata = cometmarks.GetCometMarksGlobalData();
	var observerService = Components.classes["@mozilla.org/observer-service;1"].getService(Components.interfaces.nsIObserverService); 
	
	var tick_timer = null;
	var sync_timer = null;
	var is_registered = false;
	
	this.init = function()
	{
		if (cometmarks_gdata.gObserver_Registered)
		{
			is_registered = false;
			return;
		}

		is_registered = true;
		cometmarks_gdata.gObserver_Registered = true;
		observerService.addObserver(this, "cometmarks_update_callback", false);
		observerService.addObserver(this, "cometmarks_sync_callback", false); 
		observerService.addObserver(this, "cometmarks_overwrite_callback", false); 
		begin_bookmark_listener();
		
		// begin timer
		this.tick_timer = window.setInterval("cometmarks_timetick();", 100);
		this.sync_timer = window.setInterval("cometmarks_monitor_sync();", 1000);
	};

	this.destroy = function()
	{
		if (!is_registered)
			return;
 		
		observerService.removeObserver(this, "cometmarks_update_callback");
		observerService.removeObserver(this, "cometmarks_sync_callback"); 
		observerService.removeObserver(this, "cometmarks_overwrite_callback");
		end_bookmark_listener();
			
	  cometmarks_gdata.gObserver_Registered = false;
	  observerService.notifyObservers(null, "observer_changed", null);
	    
	  window.clearInterval(this.tick_timer);
	  window.clearInterval(this.sync_timer);
	};
	
	this.observe = function(subject, topic, data)
	{
		if (topic != "cometmarks_sync_callback")
			return;
			
		var autosync_interval_sec = get_autosync_interval_sec();
		if ("OK" == data)
		{
			var bookmarks_info = cometmarks.GetCometMarksInfo();
						
			cometmarks_merge_special();					
			var server_bookmarks_content = bookmarks_info.Bookmarks_Content;  		
			if (server_bookmarks_content == "")
			{
				update_status("OK");
				write_log("sync: server bookmarks content is null");
				set_next_sync_time(autosync_interval_sec);
				return;
			}
			
			var server_revision = bookmarks_info.Bookmarks_Version;
			write_log("cometmarks current revision: " + server_revision);
			
			if (cometmarks_local_revision == server_revision)
			{
				update_status("OK");
				set_next_sync_time(autosync_interval_sec);
				return;
			}
						
			// revision not equal, and compare contents
			var local_bookmarks_content = cometmarks_access_favorite();						
			if (!cometmarks.CometmarksEqual(local_bookmarks_content))// un_equal
			{
				cometmarks_recover();
				write_log("sync: cometmarks recover completely");
			}
				
			cometmarks_local_revision = server_revision;
			update_status("OK");// include update version and lastsync_time
			set_next_sync_time(autosync_interval_sec);    		
			return;
		}
		
		if ("EXCEPTION" == data)
		{
			update_status("EXCEPTION");
			write_log("sync exception");
			cometmarks_show_error_message();
			return;
		}
		
		if ("INVALID_PASSWORD" == data)
		{
			update_status("INVALID_PASSWORD");
			write_log("sync invalid password");
			return;
		}
		
		if ("OUT_OF_DATE" == data)
		{
			update_status("OUT_OF_DATE");
			write_log("sync out of date");
			return;
		}
		
		if ("CONNECT_FAILED" == data)
		{
			update_status("CONNECT_FAILED");
			write_log("sync connect failed and begin to re-connect");
			set_next_sync_time(autosync_interval_sec);
			return;
		}
		
		if("NO_RECORD" == data)
		{
			update_status("NO_RECORD");
			write_log("sync no record");
			set_next_sync_time(autosync_interval_sec);
		}
		else
		{
			write_log("sync unknown error");
			update_status("NUKNOWN ERROR");
			cometmarks_show_error_message();
		}
	}
};

function sync_setting_status(status)
{
	var cometmarks_gdata = cometmarks.GetCometMarksGlobalData();

	var state_image = document.getElementById("general_status_image");
	var state_text = document.getElementById("general_status_static");

	var has_change = document.getElementById("general_status_has_change");
	var change_time = document.getElementById("general_status_synctime");

	var lastsync_time = document.getElementById("general_lastsync_static");

	var bubble = Components.classes["@mozilla.org/intl/stringbundle;1"]
						.getService(Components.interfaces.nsIStringBundleService)
            .createBundle("chrome://cometmarks/locale/cometmarks.properties");

	if (status == "COMETMARKS_CHANGED")
	{
		state_image.setAttribute("src", "chrome://cometmarks/skin/state_sync_pending_32.png");
		state_text.setAttribute("value", bubble.GetStringFromName("cometmarks.status.wait"));
		has_change.setAttribute("value", bubble.GetStringFromName("cometmarks.status.bookmark.changed"));
		cometmarks_gdata.gBookmarks_Changed = true;
		cometmarks_connect_failed = false;
		window.clearTimeout(timer);
		timer = window.setInterval("change_timer();",1000);
	}
	else if (status == "SYNC_SUCCESS" || status == "COMETMARKS_IDLE")
	{
		state_image.setAttribute("src", "chrome://cometmarks/skin/state_sync_ok_32.png");
		state_text.setAttribute("value", bubble.GetStringFromName("cometmarks.status.success"));
		var time = format_GMT_time(cometmarks_gdata.gLastSync_Time);
		lastsync_time.setAttribute("value", time);
		has_change.setAttribute("value", "N/A");
		cometmarks_gdata.gBookmarks_Changed = false;
		cometmarks_connect_failed = false;
		window.clearTimeout(timer);
		timer = window.setInterval("change_timer();",1000);
	}
	else if (status == "INVALID_PASSWORD")
	{
		state_image.setAttribute("src", "chrome://cometmarks/skin/state_error_32.png");
		state_text.setAttribute("value", bubble.GetStringFromName("cometmarks.status.invalid.password"));
		window.clearTimeout(timer);
		change_time.setAttribute("value", "N/A");
	}
	else if (status == "OUT_OF_DATE")
	{
		state_image.setAttribute("src", "chrome://cometmarks/skin/state_error_32.png");
		state_text.setAttribute("value", bubble.GetStringFromName("cometmarks.status.out.of.date"));
		window.clearTimeout(timer);
		change_time.setAttribute("value", "N/A");
	}
	else if (status == "CONNECT_FAILED")
	{
		state_image.setAttribute("src", "chrome://cometmarks/skin/state_error_32.png");
		state_text.setAttribute("value", bubble.GetStringFromName("cometmarks.status.connect.failed"));
		window.clearTimeout(timer);
		change_time.setAttribute("value", "N/A");
		cometmarks_gdata.gBookmarks_Changed = false;
		cometmarks_connect_failed = true;
		timer = window.setInterval("change_timer();",1000);
	}
	else if (status == "NO_RECORD")
	{
		state_image.setAttribute("src", "chrome://cometmarks/skin/state_sync_ok_32.png");
		state_text.setAttribute("value", bubble.GetStringFromName("cometmarks.status.success"));
		lastsync_time.setAttribute("value", "N/A");
		has_change.setAttribute("value", "N/A");
		cometmarks_gdata.gBookmarks_Changed = false;
		cometmarks_connect_failed = false;
		window.clearTimeout(timer);
		timer = window.setInterval("change_timer();",1000);
	}
	else if (status == "COMETMARKS_SYNCING")
	{
		state_image.setAttribute("src", "chrome://cometmarks/skin/state_connecting_32.gif");
		state_text.setAttribute("value", bubble.GetStringFromName("cometmarks.status.syncing"));
		window.clearTimeout(timer);
		change_time.setAttribute("value", "N/A");
	}
	else if (status == "COMETMARKS_UPDATING")
	{
		state_image.setAttribute("src", "chrome://cometmarks/skin/state_connecting_32.gif");
		state_text.setAttribute("value", bubble.GetStringFromName("cometmarks.status.updating"));
		window.clearTimeout(timer);
		change_time.setAttribute("value", "N/A");
	}
	else if (status == "COMETMARKS_OVERWRITING")
	{
		state_image.setAttribute("src", "chrome://cometmarks/skin/state_connecting_32.gif");
		state_text.setAttribute("value", bubble.GetStringFromName("cometmarks.status.overwriting"));
		window.clearTimeout(timer);
		change_time.setAttribute("value", "N/A");
	}
	else if (status == "UNKNOWN_STATE")
	{
		state_image.setAttribute("src", "chrome://cometmarks/skin/state_error_32.png");
		state_text.setAttribute("value", bubble.GetStringFromName("cometmarks.status.unknown.state"));
		window.clearTimeout(timer);
		change_time.setAttribute("value", "N/A");
	}
	else if (status == "EXCEPTION")
	{
		state_image.setAttribute("src", "chrome://cometmarks/skin/state_error_32.png");
		state_text.setAttribute("value", bubble.GetStringFromName("cometmarks.status.exception"));
		window.clearTimeout(timer);
		change_time.setAttribute("value", "N/A");
	}
	else if (status == "COMETMARKS_NOTLOGIN")
	{
		state_image.setAttribute("src", "chrome://cometmarks/skin/state_not_login_32.png");
		state_text.setAttribute("value", bubble.GetStringFromName("cometmarks.status.no.login"));
		lastsync_time.setAttribute("value", "N/A");
		window.clearTimeout(timer);
		change_time.setAttribute("value", "N/A");
		has_change.setAttribute("value", "N/A");
	}
}

function doLoad()
{
	// set the window type name
	document.documentElement.setAttribute("windowtype", "CometMarksSettingDialog");
	
	// hide toolbar visible property
	//window.toolbar.visible=!window.toolbar.visible;

	// general pane
	var os = Components.classes["@mozilla.org/observer-service;1"].getService(Components.interfaces.nsIObserverService);
	os.addObserver(setting_stateObserver, "cometmarks-statechange", false);
	os.addObserver(setting_stateObserver, "observer_changed", false);
	
	// register observer
	setting_register_observer.init();

	var cometmarks_gdata = cometmarks.GetCometMarksGlobalData();
	var bookmarks_info = cometmarks.GetCometMarksInfo();

	try
	{
		var ps = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService);

		var prefs = ps.getBranch("cometmarks.");

		// enable logging
		var enable_logs = prefs.getBoolPref("enable_logs");

		if (enable_logs)
			document.getElementById("advanced_enablelogging_checkbox").setAttribute("checked", "true");
		else
			document.getElementById("advanced_enablelogging_checkbox").setAttribute("checked", "false");

		// set init value
		var time = format_GMT_time(cometmarks_gdata.gLastSync_Time);
		document.getElementById("general_lastsync_static").value = time;
		var has_change = document.getElementById("general_status_has_change");
		var change_time = document.getElementById("general_status_synctime");
		has_change.setAttribute("value", "N/A");
		change_time.setAttribute("value", "N/A");
		document.getElementById("general_runwizard_button").hidden = true;
		
		// update the status
		os.notifyObservers(null, "cometmarks-statechange", cometmarks_gdata.gBookmarks_Status);
		
		// show left sync time now
		var status = cometmarks_gdata.gBookmarks_Status;
		if (status ==  "COMETMARKS_CHANGED" || status == "SYNC_SUCCESS" || status == "COMETMARKS_IDLE" 
					|| status == "CONNECT_FAILED" || status == "NO_RECORD")
		{
			change_timer();
		}

		// get username and password
		var aUsername = "", aPassword = "";
		var ps = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService);
		var prefs = ps.getBranch("cometmarks.");

		aUsername = prefs.getCharPref("username");

		var username_utf16 = utf8to16(aUsername);
		aUsername = username_utf16;
		aPassword = get_password(aUsername);// get password from loginManager
		if (aPassword != "")
			document.getElementById("general_rememberpassword_checkbox").setAttribute("checked", "true");
		else
			document.getElementById("general_rememberpassword_checkbox").setAttribute("checked", "false");
			
		if (aUsername == "")
		{
			document.getElementById("general_username_editbox").focus();
			document.getElementById("general_username_editbox").select();
			if (!bookmarks_info.Loginned)
				os.notifyObservers(null, "cometmarks-statechange", "COMETMARKS_NOTLOGIN");
			return;
		}
		else
			document.getElementById("general_username_editbox").value = aUsername;
			
		if (aPassword == "")
		{
			document.getElementById("general_password_editbox").focus();
			document.getElementById("general_password_editbox").select();
			if (!bookmarks_info.Loginned)
				os.notifyObservers(null, "cometmarks-statechange", "COMETMARKS_NOTLOGIN");
			return;
		}
		else
			document.getElementById("general_password_editbox").value = aPassword;
	}
	catch(e)
	{
		write_log(e);
	}
}

function set_remember_checkbox()
{
	document.getElementById("general_rememberpassword_checkbox").setAttribute("checked", "true");
}


function change_timer()
{
	var has_change = document.getElementById("general_status_has_change");
	var change_time = document.getElementById("general_status_synctime");
	var cometmarks_gdata = cometmarks.GetCometMarksGlobalData();

	var begin_time = null;
	if (cometmarks_connect_failed)
		begin_time = get_time_sec(cometmarks_gdata.gLastLogin_Time);
	else	
		begin_time = get_time_sec(cometmarks_gdata.gLastSync_Time);
		
	var now = new Date();
	var now_time = get_time_sec(now.toGMTString());
	if (now_time < begin_time)// i.e. now: 17:01:02  begin: 16:59:21
		now_time += 3600;
		
	var offset = now_time - begin_time;
	var autosync_interval_sec = get_autosync_interval_sec();
	var minimum_sync_interval_sec = get_minimum_sync_interval_sec();

	// when bookmarks don't change
	if (!cometmarks_gdata.gBookmarks_Changed)
	{
		if (offset > autosync_interval_sec - 1)
		{
			window.clearTimeout(timer);
			if (change_time != "undefined")
				change_time.setAttribute("value", "N/A");
			return;
		}

		var left_time = autosync_interval_sec - offset;
		var minute = "0" + Math.floor( left_time / 60);
		
		var second = "";
		if (left_time % 60 < 10)
			second += "0";
		second += left_time % 60;
		
		var left = minute + " : " + second;
		if (change_time != "undefined")
			change_time.setAttribute("value", left);
		return;
	}

	// when bookmarks changed
	if (offset > minimum_sync_interval_sec - 1 || offset < 1)
	{
		window.clearTimeout(timer);
		if (change_time != "undefined")
			change_time.setAttribute("value", "N/A");
		return;
	}

	var left = "";
	if (minimum_sync_interval_sec - offset < 10)
		left += "0";
	left += minimum_sync_interval_sec - offset;
	
	var times_str = "00 : " + left;
	if (change_time != "undefined")
		change_time.setAttribute("value", times_str);
}

// return true: close window, return false: don't close window
function doOK()
{
	try
	{
		// general pane
		var bookmarks_info = cometmarks.GetCometMarksInfo();
		var cometmarks_gdata = cometmarks.GetCometMarksGlobalData();
		var g_username = cometmarks_gdata.gUserName;
		var g_password = cometmarks_gdata.gPassword;
		
		var aPassword = document.getElementById("general_password_editbox").value;
		var aUsername = document.getElementById("general_username_editbox").value;
		
		var ps = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService);
	  var prefs = ps.getBranch("cometmarks.");
	  
		var bubble = Components.classes["@mozilla.org/intl/stringbundle;1"]
									.getService(Components.interfaces.nsIStringBundleService)
									.createBundle("chrome://cometmarks/locale/cometmarks.properties");
		var message = "";
		
		var os = Components.classes["@mozilla.org/observer-service;1"].getService(Components.interfaces.nsIObserverService);
		if (aUsername == "" && aPassword == "") // both username and password are null
		{
			if (!bookmarks_info.Loginned)// no login
			{			
				// clear accout's info
				cometmarks.LogOut();
				clear_accout_data();
				set_next_sync_time("N/A");
				
				os.notifyObservers(null, "cometmarks-statechange", "COMETMARKS_NOTLOGIN");
				cometmarks_gdata.gBookmarks_Status = "COMETMARKS_NOTLOGIN";
				return true;
			}
			
			message = bubble.GetStringFromName("cometmarks.sign.out");	
			if (window.confirm(message))// has logined and want to exit
			{ 
				// if overwrite dialog exist, close it first
				var overwrite_win = get_opened_dialog("CometMarksOverwriteDialog");
				if (overwrite_win)// has opened
					overwrite_win.close();
				
				// logout and clear accout's info
				cometmarks.LogOut();
				clear_accout_data();
				set_next_sync_time("N/A");
				
				// set status as no login
				os.notifyObservers(null, "cometmarks-statechange", "COMETMARKS_NOTLOGIN");
				cometmarks_gdata.gBookmarks_Status = "COMETMARKS_NOTLOGIN";
				
				// close note panel
				os.notifyObservers(null, "cometmarks-notes-state", "close");
				os.notifyObservers(null, "cometmarks-notes-state", "disabled");
	    	
	    	return true;
			}
			else // cancel, accout doesn't exit login
			{
				return false;
			}
		}
		
		if (aUsername == "")
		{
			message = bubble.GetStringFromName("cometmarks.sign.username");
			alert(message);
	   	document.getElementById("general_username_editbox").focus();
	    document.getElementById("general_username_editbox").select();
	    return false;
		}
	
		if (aPassword == "")
		{				
			message = bubble.GetStringFromName("cometmarks.sign.password");
		  alert(message);
		  document.getElementById("general_password_editbox").focus();
		  document.getElementById("general_password_editbox").select();
		  return false;
		}
	
		if (g_username == aUsername && g_password == aPassword)
		{
			// if overwrite dialog exist, focus it
			var overwrite_win = get_opened_dialog("CometMarksOverwriteDialog");
			if (overwrite_win)// has opened
			{
				overwrite_win.focus();
				return true;// close setting dialog
			}
			
			if (bookmarks_info.Loginned)
			{
				cometmarks_sync();
			}
			else
			{
				write_log("re-login account: " + aUsername);
				cometmarks_login();
			}
		}
		else
		{
			// if overwrite dialog exist, close it first
			var overwrite_win = get_opened_dialog("CometMarksOverwriteDialog");
			if (overwrite_win)// has opened
			{
				overwrite_win.close();
				cometmarks_gdata.gMerge_Closed_Directly = true;
				cometmarks_gdata.gBookmarks_Changed = true;
				
				// logout and clear accout data
				cometmarks.LogOut();
				clear_accout_data();
				set_next_sync_time("N/A");
			}
	
			// save username 
			cometmarks_gdata.gUserName = aUsername;
			var username_utf8 = utf16to8(aUsername);
			prefs.setCharPref("username", username_utf8);
	  	
			// close note panel
			os.notifyObservers(null, "cometmarks-notes-state", "close");
			os.notifyObservers(null, "cometmarks-notes-state", "disabled");
				
			// change statusbar username
			write_log("change login account: " + aUsername);
			cometmarks_set_username(aUsername, aPassword);
		}
	  
	  // save password
		if (document.getElementById("general_rememberpassword_checkbox").checked)
		{
		  add_accout(aUsername, aPassword);// add/modify account to loginManager
		  cometmarks_gdata.gPassword = aPassword;
		}
		else
		{
		 add_accout(aUsername, "");// delete password
		 cometmarks_gdata.gPassword = "";
		}
	
	  // enable logging
	  if (document.getElementById("advanced_enablelogging_checkbox").checked)
	  	prefs.setBoolPref("enable_logs", true);
	  else
	  	prefs.setBoolPref("enable_logs", false);
	
	  return true;
	}
	catch (e)
	{
		write_log(e);
	}
}


function update_status(message)
{
	try
	{
		var os = Components.classes["@mozilla.org/observer-service;1"].getService(Components.interfaces.nsIObserverService);
		if (null == cometmarks)
		{
			write_log("cometmarks == null");
			return;
		}

		var bookmarks_info = cometmarks.GetCometMarksInfo();
		var cometmarks_gdata = cometmarks.GetCometMarksGlobalData();
		var state = bookmarks_info.Current_State;
		if (message == "OK")
		{		
			// set last sync time
			var str_now_time = new Date();
			cometmarks_gdata.gLastSync_Time = str_now_time.toGMTString();
			write_log("the last sync date: " + format_GMT_time(str_now_time) );
			write_log("---finish log this time---\r");
			
			cometmarks_gdata.gBookmarks_Status = state;
			if ("COMETMARKS_UPDATING" == state)
				os.notifyObservers(null, "cometmarks-statechange", "COMETMARKS_UPDATING");
			else if ("COMETMARKS_SYNCING" == state)
				os.notifyObservers(null, "cometmarks-statechange", "COMETMARKS_SYNCING");
			else if ("COMETMARKS_IDLE" == state)
				os.notifyObservers(null, "cometmarks-statechange", "SYNC_SUCCESS");
			else if ("UNKNOWN_STATE" == state)
				os.notifyObservers(null, "cometmarks-statechange", "UNKNOWN_STATE");
			else if ("COMETMARKS_OVERWRITING" == state)
				os.notifyObservers(null, "cometmarks-statechange", "COMETMARKS_OVERWRITING");
		}
		else if (message == "EXCEPTION")
		{
			os.notifyObservers(null, "cometmarks-statechange", "EXCEPTION");
			write_log(message);
		}
		else if (message == "INVALID_PASSWORD")
		{
			os.notifyObservers(null, "cometmarks-statechange", "INVALID_PASSWORD");
			write_log(message);
		}
		else if (message == "OUT_OF_DATE")
		{
			os.notifyObservers(null, "cometmarks-statechange", "OUT_OF_DATE");
			write_log(message);
		}
		else if (message == "CONNECT_FAILED")
		{
			var str_now_time = new Date();
			cometmarks_gdata.gLastLogin_Time = str_now_time.toGMTString();
			os.notifyObservers(null, "cometmarks-statechange", "CONNECT_FAILED");
			write_log(message);
		}
		else if (message == "NO_RECORD")
		{
			var str_now_time = new Date();
			cometmarks_gdata.gLastSync_Time = str_now_time.toGMTString();
			os.notifyObservers(null, "cometmarks-statechange", "NO_RECORD");
			write_log(message);
		}
	}
	catch(e)
	{
		write_log(e);
	}
}

function cometmarks_get_forget_password()
{
	var url = document.getElementById("general_get_forget_password").crop;// get forget password url store in crop attribute
	open_url(url);
}


function cometmarks_register_accout()
{
	var url = document.getElementById("general_register_accout").crop;// get register accout url store in crop attribute
	open_url(url);
}

function Un_load()
{
	var os = Components.classes["@mozilla.org/observer-service;1"].getService(Components.interfaces.nsIObserverService);
	if (setting_stateObserver != null && setting_stateObserver != undefined)
	{
		os.removeObserver(setting_stateObserver, "cometmarks-statechange");
		os.removeObserver(setting_stateObserver, "observer_changed");	
	}
	
	// remove observer
	setting_register_observer.destroy();
}